#include "student_points.hpp"

template <GradingType Grading>
void StudentPoints<Grading>::recalculate() {

    const auto &examRef = std::get<const DynexiteExam &>(this->mParentRefs);
    const auto &bonusRef =
        std::get<const AppliedBonusPoints<Grading> &>(this->mParentRefs);

    // Resize the QList, reset the points
    mPointsWithBonus =
        QList<Points>(examRef.mPointsWithoutBonus[Grading].size());

    // Generate a list of grades in the same order the points were in
    for (qsizetype i = 0; i < mPointsWithBonus.size(); ++i) {

        mPointsWithBonus[i] = examRef.mPointsWithoutBonus[Grading][i] +
                              bonusRef.getNrBonusPoints(examRef.mIdentifier[i]);
    }
}

void CombinedStudentPoints::recalculate() {

    const auto &closedRef =
        std::get<const StudentPoints<GradingType::Closed> &>(this->mParentRefs);
    const auto &openRef =
        std::get<const StudentPoints<GradingType::Open> &>(this->mParentRefs);

    mPointsWithBonus.clear();
    std::ranges::transform(closedRef.mPointsWithBonus, openRef.mPointsWithBonus,
                           std::back_inserter(mPointsWithBonus), std::plus());
}

template struct StudentPoints<GradingType::Closed>;
template struct StudentPoints<GradingType::Open>;
