#include "data/nodes/passing_limit.hpp"

#include <gtest/gtest.h>

TEST(PassingLimitsTest, absPassingLimit) {

    const GradeBoundaryFactory f(GradeBoundaryStepSize::Min1_00,
                                 GradeBoundaryRounding::Up);

    // Handreichung Anlage 1 Beispiel 1-3
    EXPECT_EQ(impl::calculateAbsolutePassingLimit(
                  f(Points{90.}), GradeBoundaryStepSize::Min1_00),
              Points{54.});

    EXPECT_EQ(impl::calculateAbsolutePassingLimit(
                  f(Points{120.}), GradeBoundaryStepSize::Min1_00),
              Points{72.});

    EXPECT_EQ(impl::calculateAbsolutePassingLimit(
                  f(Points{40.}), GradeBoundaryStepSize::Min1_00),
              Points{24.});
}
TEST(PassingLimitsTest, relPassingLimit2) {
    // Handreichung Anlage 1 Beispiel 1-3

    const GradeBoundaryFactory f(GradeBoundaryStepSize::Min1_00,
                                 GradeBoundaryRounding::Up);

    const QList firstTryPoints1{Points{66.}, Points{67.}, Points{68.}};
    const QList firstTryPoints2{Points{73.}};
    const QList firstTryPoints3{Points{32.}, Points{34.}};

    // for some reason MSVC gives internal compiler errors if we call these
    // functions in the gtest-macro.
    const auto res1 = impl::calculateRelativePassingLimit(
        f(Points{90.}), GradeBoundaryStepSize::Min1_00, firstTryPoints1);
    const auto res2 = impl::calculateRelativePassingLimit(
        f(Points{120.}), GradeBoundaryStepSize::Min1_00, firstTryPoints2);
    const auto res3 = impl::calculateRelativePassingLimit(
        f(Points{40.}), GradeBoundaryStepSize::Min1_00, firstTryPoints3);

    EXPECT_NEAR(res1.p, 52.26, 0.005);
    EXPECT_EQ(res2.p, 60.);
    EXPECT_NEAR(res3.p, 25.74, 0.005);

    // empty list of first try points
    const auto res4 = impl::calculateRelativePassingLimit(
        f(Points{41.}), GradeBoundaryStepSize::Min1_00, {});
    EXPECT_EQ(res4, impl::calculateAbsolutePassingLimit(
                        f(Points{41.}), GradeBoundaryStepSize::Min1_00));
}

TEST(PassingLimitsTest, passingLimit) {
    // Handreichung Anlage 1 Beispiel 1-3

    const GradeBoundaryFactory f(GradeBoundaryStepSize::Min1_00,
                                 GradeBoundaryRounding::Up);

    const QList firstTryPoints1{Points{66.}, Points{67.}, Points{68.}};
    const QList firstTryPoints2{Points{73.}};
    const QList firstTryPoints3{Points{32.}, Points{34.}};

    // for some reason MSVC gives internal compiler errors if we call these
    // functions in the gtest-macro.
    const auto res1 = impl::calculatePassingLimit(
        f(Points{90.}), GradeBoundaryStepSize::Min1_00, firstTryPoints1);
    const auto res2 = impl::calculatePassingLimit(
        f(Points{120.}), GradeBoundaryStepSize::Min1_00, firstTryPoints2);
    const auto res3 = impl::calculatePassingLimit(
        f(Points{40.}), GradeBoundaryStepSize::Min1_00, firstTryPoints3);

    // Handreichung Anlage 1 Beispiel 1
    EXPECT_NEAR(res1.p, 52.26, 0.005);

    // Handreichung Anlage 1 Beispiel 2
    EXPECT_EQ(res2.p, 60.);

    // Handreichung Anlage 1 Beispiel 3
    EXPECT_NEAR(res3.p, 24., 0.005);

    // empty list of first try points
    const auto res4 = impl::calculatePassingLimit(
        f(Points{43.}), GradeBoundaryStepSize::Min1_00, {});
    EXPECT_EQ(res4, impl::calculateAbsolutePassingLimit(
                        f(Points{43.}), GradeBoundaryStepSize::Min1_00));
}
